package cz.jzitnik;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.ui.*;
import cz.jzitnik.tui.MouseHandler;
import cz.jzitnik.game.SpriteLoader;
import cz.jzitnik.tui.ScreenRenderer;
import org.jline.terminal.MouseEvent;
import org.jline.terminal.Terminal;
import org.jline.terminal.TerminalBuilder;

import java.io.IOException;
import java.util.Optional;


public class Main {
    public static void main(String[] args) {
        try {
            Terminal terminal = TerminalBuilder.terminal();
            terminal.enterRawMode();
            terminal.trackMouse(Terminal.MouseTracking.Any);

            if (!terminal.hasMouseSupport()) {
                System.out.println("Error: This terminal does not support mouse.");
                System.exit(1);
            }

            var spriteList = SpriteLoader.load();
            var screenRenderer = new ScreenRenderer(spriteList, terminal);
            var game = new Game();

            MouseHandler mouseHandler = new MouseHandler(game, terminal, screenRenderer);

            final boolean[] isRunning = {true};

            Thread inputThread = new Thread(() -> {
                try {
                    while (isRunning[0]) {
                        int key = terminal.reader().read();

                        // Check for mouse event sequence
                        if (key == 27) { // ESC
                            key = terminal.reader().read();
                            if (key == '[') {
                                key = terminal.reader().read();
                                if (key == 'M') {
                                    MouseEvent mouseEvent = terminal.readMouseEvent();
                                    switch (game.getWindow()) {
                                        case WORLD -> mouseHandler.handle(mouseEvent);
                                        case INVENTORY -> InventoryClickHandler.click(mouseEvent, terminal, screenRenderer, game, Optional.empty(), Optional.empty());
                                        case CRAFTING_TABLE -> game.getGameStates().craftingTable.click(mouseEvent, terminal, screenRenderer);
                                        case CHEST -> ((Chest) game.getWorld()[game.getGameStates().clickY][game.getGameStates().clickX].stream().filter(i -> i.getBlockId().equals("chest")).toList().getFirst().getData()).click(game, mouseEvent, terminal, screenRenderer);
                                        case FURNACE -> ((Furnace) game.getWorld()[game.getGameStates().clickY][game.getGameStates().clickX].stream().filter(i -> i.getBlockId().equals("furnace")).toList().getFirst().getData()).click(game, mouseEvent, terminal, screenRenderer);
                                    }
                                }
                            }
                        }

                        switch (key) {
                            case '1':
                            case '2':
                            case '3':
                            case '4':
                            case '5':
                            case '6':
                            case '7':
                            case '8':
                            case '9':
                                game.changeSlot(key - 49, screenRenderer);
                                break;
                            case 'a':
                                game.movePlayerLeft(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case 'd':
                                game.movePlayerRight(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case ' ':
                                game.movePlayerUp(screenRenderer);
                                screenRenderer.render(game);
                                break;
                            case 'e':
                                if (game.getWindow() != Window.WORLD) {
                                    game.getInventory().setSelectedItemInv(-1);
                                    CloseHandler.handle(game.getWindow(), game);
                                    game.setWindow(Window.WORLD);
                                } else {
                                    game.setWindow(Window.INVENTORY);
                                }
                                screenRenderer.render(game);
                                break;
                            case 'q':
                                System.out.println("Exiting game...");
                                isRunning[0] = false;
                                break;
                            default:
                                break;
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });

            inputThread.start();

            // Game loop (rendering the game)
            while (isRunning[0]) {
                if (game.getWindow() == Window.WORLD) {
                    // Rerender the game only when default window
                    screenRenderer.render(game);
                }
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            // Disable mouse tracking on exit
            terminal.trackMouse(Terminal.MouseTracking.Off);

            terminal.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
