package cz.jzitnik.game.items;

import cz.jzitnik.game.SpriteLoader;
import cz.jzitnik.game.Block;
import cz.jzitnik.game.ui.Chest;
import cz.jzitnik.game.ui.Furnace;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ItemBlockSupplier {
    // This is necessary to not get infinite recursion
    private static class Helper {
        public static Item dirt(Block ref) {
            return new Item("dirt", "Dirt block", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_DIRT, ref);
        }
        public static Item cobblestone(Block ref) {
            return new Item("cobblestone", "Cobblestone", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_COBBLESTONE, ref);
        }
        public static Item stone(Block ref) {
            return new Item("stone", "Stone", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_STONE, ref);
        }
        public static Item oakLog(Block ref) {
            return new Item("oak_log", "Oak log", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_OAK_LOG, ref);
        }
        public static Item oakPlanks(Block ref) {
            return new Item("oak_planks", "Oak planks", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_OAK_PLANKS, ref);
        }
        public static Item craftingTable(Block ref) {
            return new Item("crafting_table", "Crafting table", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_CRAFTING_TABLE, ref);
        }
        public static Item chest(Block ref) {
            return new Item("chest", "Chest", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_CHEST, ref);
        }
        public static Item furnace(Block ref) {
            return new Item("furnace", "Furnace", ItemType.BLOCK, SpriteLoader.SPRITES.ITEM_FURNACE, ref);
        }
    }

    public static class Blocks {
        public static Block dirt() {
            var block = new Block("dirt", SpriteLoader.SPRITES.DIRT, 1, ItemType.SHOVEL, new ArrayList<>());
            block.setDrops(List.of(Helper.dirt(block)));
            return block;
        }
        public static Block grass() {
            var block = new Block("grass", SpriteLoader.SPRITES.GRASS, 1, ItemType.SHOVEL, new ArrayList<>());
            block.setDrops(List.of(Items.dirt()));
            return block;
        }
        public static Block cobblestone() {
            var block = new Block("cobblestone", SpriteLoader.SPRITES.COBBLESTONE, 15, ItemType.PICKAXE, Arrays.stream(ToolVariant.values()).toList());
            block.setDrops(List.of(Helper.cobblestone(block)));
            return block;
        }
        public static Block stone() {
            var block = new Block("stone", SpriteLoader.SPRITES.STONE, 15, ItemType.PICKAXE, Arrays.stream(ToolVariant.values()).toList());
            block.setDrops(List.of(Items.cobblestone()));
            return block;
        }
        public static Block oakLog() {
            var block = new Block("oak_log", SpriteLoader.SPRITES.OAK_LOG, 3, ItemType.AXE, new ArrayList<>());
            block.setDrops(List.of(Helper.oakLog(block)));
            return block;
        }
        public static Block oakPlanks() {
            var block = new Block("oak_planks", SpriteLoader.SPRITES.OAK_PLANKS, 3, ItemType.AXE, new ArrayList<>());
            block.setDrops(List.of(Helper.oakPlanks(block)));
            return block;
        }
        public static Block craftingTable() {
            var block = new Block("crafting_table", SpriteLoader.SPRITES.CRAFTING_TABLE, 3, ItemType.AXE, new ArrayList<>());
            block.setDrops(List.of(Helper.craftingTable(block)));
            return block;
        }
        public static Block chest() {
            var block = new Block("chest", SpriteLoader.SPRITES.CHEST, 3, ItemType.AXE, new ArrayList<>(), new Chest());
            block.setDrops(List.of(Helper.chest(block)));
            return block;
        }
        public static Block furnace() {
            var block = new Block("furnace", SpriteLoader.SPRITES.FURNACE, 15, ItemType.PICKAXE, Arrays.stream(ToolVariant.values()).toList());
            block.setData(new Furnace(block));
            block.setDrops(List.of(Helper.furnace(block)));
            return block;
        }
    }

    // I hate this but whatever
    public static class Items {
        public static Item dirt() {
            return Helper.dirt(Blocks.dirt());
        }
        public static Item oakPlanks() {
            return Helper.oakPlanks(Blocks.oakPlanks());
        }
        public static Item stick() {
            return new Item("stick", "Stick", ItemType.USELESS_ITEM, SpriteLoader.SPRITES.ITEM_STICK);
        }
        public static Item craftingTable() {
            return Helper.craftingTable(Blocks.craftingTable());
        }
        public static Item woodenPickaxe() {
            return new Item("wooden_pickaxe", "Wooden pickaxe", ItemType.PICKAXE, SpriteLoader.SPRITES.WOODEN_PICKAXE, ToolVariant.WOODEN, 12, 59, false);
        }
        public static Item woodenAxe() {
            return new Item("wooden_axe", "Wooden axe", ItemType.AXE, SpriteLoader.SPRITES.WOODEN_AXE, ToolVariant.WOODEN, 2, 59, false);
        }
        public static Item cobblestone() {
            return Helper.cobblestone(Blocks.cobblestone());
        }
        public static Item stonePickaxe() {
            return new Item("stone_pickaxe", "Stone pickaxe", ItemType.PICKAXE, SpriteLoader.SPRITES.STONE_PICKAXE, ToolVariant.STONE, 12.5, 132, false);
        }
        public static Item stoneAxe() {
            return new Item("stone_axe", "Stone axe", ItemType.AXE, SpriteLoader.SPRITES.STONE_AXE, ToolVariant.STONE, 3, 132, false);
        }
        public static Item woodenShovel() {
            return new Item("wooden_shovel", "Wooden shovel", ItemType.SHOVEL, SpriteLoader.SPRITES.WOODEN_SHOVEL, ToolVariant.WOODEN, 0.3, 59, false);
        }
        public static Item chest() {
            return Helper.chest(Blocks.chest());
        }
        public static Item stoneShovel() {
            return new Item("stone_shovel", "Stone shovel", ItemType.SHOVEL, SpriteLoader.SPRITES.STONE_SHOVEL, ToolVariant.STONE, 0.5, 132, false);
        }
        public static Item stone() {
            return Helper.stone(Blocks.stone());
        }
        public static Item furnace() {
            return Helper.furnace(Blocks.furnace());
        }
    }
}
